#!/bin/sh
#------------------------------------------------------------------------------
# /usr/bin/dyndns-update.sh - Update one DynDNS-provider
# part of Package DYNDNS 3.10.5, see documentation for licence
#
#  Copyright 2003-2004 Tobias Gruetzmacher
#
# Creation:    16.09.2003 tobig
# Last Update: $Id: dyndns-update.sh 34679 2014-11-06 23:25:01Z kristov $
#------------------------------------------------------------------------------

#
# Error codes
#
#   connection related errors
ECONN_CONN=1
ECONN_PERM=2
#
#   provider related errors
#
EPROV_TMP=11
EPROV_PERM=12
EPROV_EMPTY=13
EPROV_UNKNOWN=14
#
#   packet related errors
#
EPACK_NOPATTERN=21
EPACK_INVFUNC=22
#
#   try related errors
ETRY_TMP=31
ETRY_PERM=32
ETRY_NOMETHOD=33

abort ()
{
    echo "$prog: $1"
    echo "Aborting!"
    exit 1
}

usage ()
{
    cat <<EOF
Usage:
    $prog update        <hostname> [ <circuit> ]
    $prog enable        <hostname>
    $prog enable_method <hostname> <method>
    $prog cancel_update <hostname>
    $prog status        <hostname>

Additional options:
    -x        : generate trace in /var/tmp/dyndns-*
    -l <lock> : use file <lock> to guard against parallel execution  
EOF
    exit 1
}

comment ()
{
    echo "# DynDNS-Comment: $1"
}

dyndns_date ()
{
    date
}

dyndns_log ()
{
    logger -t "dyndns-update $host [$$]" "$1"
}

dyndns_out ()
{
    if [ "$syslog" = yes ]; then
        dyndns_log "$1"
    else
        echo "dyndns-update.sh: $1"
    fi
}
#
# This is a plain http-request with or without basic auth
#
http_update ()
{
    for port in $provider_port; do
        {
            echo "GET $provider_url HTTP/1.1"
            echo "Host: $provider_host"
            if [ "$provider_auth" ]; then
                echo "Authorization: Basic `echo -n "$provider_auth" | base64`"
            fi
            echo "User-Agent: dyndns_fli4l/$dyn_ver fli4l@null.bogus"
            echo ""
        } | netcat -w "$dyn_timeout" "$provider_host" "$port"
        if [ $? -eq 0 ]; then
            return 0
        fi
    done
    return $ECONN_CONN
}

#
# This is a https-request using curl
#
https_update ()
{
    if [ "$1" = "noauth" ]; then
        cert="--insecure"
    else
        cert="--cacert /usr/share/curl/${cacert:=ca-bundle.crt}"
    fi

    curl --connect-timeout "$dyn_timeout" \
        --get --user-agent "dyndns_fli4l/$dyn_ver fli4l@null.bogus" \
        $cert --tlsv1 \
        ${curl_line}

    case $? in
        0) return 0 ;;
        7 | 28)
            # temporary failure, other methods still might work
            # 7  - Failed to connect to host.
            # 28 - Operation timeout. The specified time-out period 
            #      was reached according to the conditions.
            return $ECONN_CONN
            ;;
        *)  return $ECONN_PERM ;;
    esac
}

#
# Use GnuDip over HTTP (Ask me if you need the direct socket connection)
#
gnudip_update ()
{
    tmpfile="/tmp/dyndns.$$.gnudip"
    {
        echo "GET $provider_script HTTP/1.1"
        echo "Host: $provider_host"
        if [ "$provider_auth" ]; then
            echo "Authorization: Basic `echo -n "$provider_auth" | base64`"
        fi
        echo "Connection: close"
        echo "User-Agent: opt_dyndns/$dyn_ver fli4l@null.bogus"
        echo ""
    } | netcat -w "$dyn_timeout" "$provider_host" "$provider_port" > $tmpfile
    if [ $? -ne 0 ]; then
        rm -f $tmpfile
        return $ECONN_CONN
    fi

    #
    # Sets meta_salt, meta_time and meta_sign
    #
    eval "`sed -n 's/^<meta name="\([A-Za-z]*\)" content="\([A-Za-z0-9]*\)">.*$/meta_\1='"'\2'"'/; /^meta_/ p' $tmpfile`"
    rm -f $tmpfile

    set -- `echo -n "$pass" | md5sum`
    md5pass="$1"
    set -- `echo -n "$md5pass"."$meta_salt" | md5sum`
    endpass="$1"

    {
        echo "GET $provider_script?salt=$meta_salt&time=$meta_time&sign=$meta_sign&user=$user&pass=$endpass&domn=$domain&reqc=2 HTTP/1.1"
        echo "Host: $provider_host"
        if [ "$provider_auth" ]; then
            echo "Authorization: Basic `echo -n "$provider_auth" | base64`"
        fi
        echo "Connection: close"
        echo "User-Agent: opt_dyndns/$dyn_ver fli4l@null.bogus"
        echo ""
    } | netcat -w "$dyn_timeout" "$provider_host" "$provider_port"
    if [ $? -ne 0 ]; then
        return $ECONN_CONN
    fi
    return 0
}

netcat_update ()
{
    for port in $provider_port; do
        if echo -e "$provider_data" |\
            netcat -w "$dyn_timeout" "$provider_host" "$port"
        then
            return 0 
        fi
    done
    return $ECONN_CONN
}

nsupdate_update ()
{
    {
	echo "server $updatehost"
        echo "update delete $host A"
	echo "update add $host 60 A $ip"
	echo "send"
        echo "quit"
    } | nsupdate -y "$user":"$pass"
    if [ $? -eq 0 ]; then
	echo "success"
        return 0
    fi
    echo "fail"
    return $EPROV_UNKNOWN
}

pattern_available ()
{
    eval success="\$${1}_success"

    if [ "$success" ]; then
        eval failure="\$${1}_failure"
        return 0
    fi

    return 1
}
protocol_pattern_available ()
{
    eval prot="\$${1}_protocol"
    [ "$prot" ] || return 1

    pattern_available ${prot}_protocol
}

get_result_patterns ()
{
    local method=$1
    case $method in
        https_noauth) 
            alternate=provider_https
            alternate2=provider_http 
            ;;
        https) 
            alternate=provider_http
            ;;
            
        *)  alternate= ;;
    esac

    for name in provider_${method} $alternate $alternate2 provider; do
        pattern_available $name && return 0
    done

    . /etc/dyndns/provider.patterns
    for name in provider_${method} $alternate $alternate2 provider; do
        protocol_pattern_available $name && return 0
    done

    pattern_available ${method}_protocol && return 0
    
    return 1
}

decode_file ()
{
    echo >> $1
    if grep -q '^Transfer-Encoding: chunked' $1; then
        sed -e '${/^[[:space:]]*$/d}' $1 | decode_chunked
    else
        sed -e '${/^[[:space:]]*$/d}' $1
    fi
}

grep_meta_info ()
{   
    sed -n '/<meta/ {
s/\&/\&amp;/g
s/</\&lt;/g
s/>/\&gt;/g
p
}'
}

strip_html_header ()
{
    sed -e "/<.DOCTYPE/!bb
:a
/<body>/!{
N
ba
}
s:\(<.DOCTYPE.*<body>\):<!--\n\1\n-->:

:b

/<\/body/!bd
:c
/<\/html>/!{
N
bc
}
s:\(</body>.*</html>\):<!--\n\1\n-->:
:d

/^Content-Type.*/r $1"
}

keep_copy () {
    local arch=$dyn_logdir/archive
    local arch_copy=$arch/${1##*/}.`date +%d.%m.%Y`
    [ -d $arch ] || mkdir -p $arch
    cp $1 $arch_copy
    comment "Can not decide, whether this update was ok or not - considering it as ok."
    comment "Success: '$success', Failure: '$failure'"
    echo "# DynDNS-Report: $arch_copy"
}

try_method ()
{
    local method=$1
    local sim=

    echo "# DynDNS-Method: '$method'"
    echo -n "# DynDNS-Date: "
    dyndns_date

    eval sim=\$provider_${method}_sim_error
    [ "$sim" ] && return $sim

    if ! get_result_patterns $method; then
        comment "Missing result patterns or protocol specification, aborting."
        return $EPACK_NOPATTERN
    else
        case "$success" in
            '|'*)
                ${success#|} /dev/null 2> /dev/null
                if [ $? -eq 127 ]; then
                    comment "Invalid check function for method '$method' - '${success#|}'"
                    return $EPACK_INVFUNC
                fi
                ;;
        esac

    fi

    #
    # contact provider
    #
    {
        case "$method" in
            http | https | gnudip | netcat | nsupdate) ${method}_update ;;
            https_noauth) https_update noauth;;
            *) comment "Unknown update method '$method'"
               false
               ;;
        esac
        res=$?
    } > $dyn_trylog 2> $dyn_trylog.err

    if [ $res -eq 0 ]; then
        if grep -q -v '^[[:space:]]*$' $dyn_trylog; then
            #
            # check for success
            #
            case "$success" in
            no_check)
                #
                # do not check at all
                #
                res=$EPROV_UNKNOWN
                ;;
            none)
                #
                # success pattern unknown, only check failure pattern
                #
                if grep -q "$failure" $dyn_trylog; then
                    res=$EPROV_PERM
                else
                    res=$EPROV_UNKNOWN
                fi
                ;;
            '|'*)
                # 
                # complex check, invoke check function
                #
                ${success#|} $dyn_trylog
                res=$?
                ;;
            *)
                #
                # success and failure pattern available, check success first, then failure
                #
                if  grep -q "$success" $dyn_trylog; then
                    res=0
                elif grep -q "$failure" $dyn_trylog; then
                    res=$EPROV_PERM
                else
                    res=$EPROV_UNKNOWN
                fi
                ;;
            esac
        else
            res=$EPROV_EMPTY
        fi
        if [ $res -eq 0 ]; then
            type=answer
        else
            type=error
        fi
        echo "----- start $type -----"
        if grep -q "Content-Type:.*text/html" $dyn_trylog; then
            sed_tmp=/tmp/dyndns.sed.$$
            grep_meta_info < $dyn_trylog > $sed_tmp
            decode_file  $dyn_trylog | strip_html_header $sed_tmp
            rm -f $sed_tmp
        else
            decode_file  $dyn_trylog
        fi
        echo "------ end $type ------"
    else
        echo "----- start error -----"
        cat $dyn_trylog.err            
        echo "------ end error ------"
    fi
    return $res
}

disable_method ()
{
    provider_update_method=`echo "$provider_update_method" | sed -e "s/$1\([[:space:]]\+\|$\)//g"`
}

try_update ()
{
    for method in $provider_update_method; do
        if [ -f $dyn_disabled.$method ]; then
            disable_method $method
            continue
        fi
        while true; do
            try_method $method
            # errors
            #   ECONN_CONN, ECONN_PERM, 
            #   EPROV_TMP, EPROV_PERM, EPROV_EMPTY, EPROV_UNKNOWN
            #   EPACK_NOPATTERN, EPACK_INVFUNC

            case $? in
                0)
                    return 0
                    ;;
                $ECONN_CONN)
                    # connection problem - try one of the other methods
                    comment 'Failed to connect provider.'
                    break
                    ;;
                $ECONN_PERM)
                    # connection problem, disable method
                    comment "Permanent connection error, disabling method '$method'."
                    cp $dyn_trylog.err $dyn_disabled.$method 
                    disable_method $method
                    break
                    ;;
                $EPROV_TMP)
                    # temporary problem, try again after some time
                    comment "Temporary problem, re-trying again after timeout (${dyn_method_timeout}s)."
                    sleep $dyn_method_timeout
                    continue
                    ;;
                $EPROV_PERM)
                    # permanent error, stop updating account, until 
                    # the problem is fixed
                    comment "Permanent error, disabling host '$host'."
                    cp $dyn_trylog.err $dyn_disabled
                    return $ETRY_PERM
                    ;;
                $EPROV_EMPTY)
                    # empty reply from provider, try other method
                    comment 'Empty reply from provider.'
                    break
                    ;;
                $EPROV_UNKNOWN)
                    # can not decide whether this was ok or not, consider this to be ok
                    keep_copy $dyn_trylog
                    return 0
                    ;;
                $EPACK_NOPATTERN | $EPACK_INVFUNC)
                    # packet problem, disable method, try another one
                    comment "Packet error, check pattern or check function missing or broken, disabling method '$method'."
                    cp $dyn_trylog.err $dyn_disabled.$method 
                    disable_method $method
                    break
                    ;;
                *)
                    # permanent error, stop updating account, until 
                    # the problem is fixed
                    comment 'Unkown error, disabling host.'
                    cp $dyn_trylog.err $dyn_disabled
                    return $ETRY_PERM
                    ;;
            esac
        done
    done
    if [ ! "$provider_update_method" ]; then
        comment "All methods disabled, no method left to try."
        return $ETRY_NOMETHOD
    fi
    return $ETRY_TMP
}

aquire_lock ()
{
    #
    # update already running?
    #
    if [ -f $dyn_pid_file ]; then
        read pid < $dyn_pid_file
        if kill -0 $pid 2> /dev/null; then
            # kill whole process group
            dyndns_out "Update still running (pid $pid), terminating previous update ..."
            pg=`cut -d' ' -f 5 /proc/$pid/stat`
            [ "$pg" ] && kill -TERM -$pg 2> /dev/null
        else
            dyndns_out "ignoring stale pid file (pid $pid) ..."
        fi
    fi
    echo $$ > $dyn_pid_file
}

get_external_ip ()
{
    extip_service=$1
    circuit=$2
    case "$extip_service" in
    yes|dyndns)
        ip=`echo -e "GET / HTTP/1.1\r\nHost: checkip.dyndns.org\r\nConnection: close\r\n\r\n" | \
            netcat -w "$dyn_timeout" checkip.dyndns.org 80 | \
            sed -n -e 's/.*Current IP Address:[[:space:]]*\([0-9.]\+\).*/\1/p'`
        ;;
    stun)
	ip=`get_ext_ip_via_stun $circuit`
        ;;
    esac
}

generic_get_registered_ip ()
{
    set -- `nslookup $host | sed -n -e '/^Server:/,/^Address/d;s/^Address[^:]*://p'`
    [ "$1" ] && res=$1
}

check_dyn_runfile ()
{
    res=unknown
    if [ -f /etc/dyndns/lookup_names ]; then
        eval local get_ip=\$${provider}_get_ip
        : ${get_ip:=generic_get_registered_ip}
        $get_ip
    fi
    echo -e "registered_ip='$res'\nregistered_date='`date +%s`'\nregistered_checked_only=yes" > $dyn_runfile
}

check_update_needed ()
{
    #
    # update actually needed?
    #
    [ -f $dyn_runfile ] || check_dyn_runfile
    [ -f $dyn_runfile ] || return

    do_update=yes
    . $dyn_runfile
    if [ -z "$dyn_force_always" -a "$ip" = "$registered_ip" ]; then
        do_update=no
        time=`date +%s`
        if [ $time -gt $registered_date  ]; then
            time_diff=`expr $time - $registered_date`
            #
            # make renew time configurable
            #
            if [ ! "$dyn_force" ]; then
                : ${renew_days:=29}
                renew_time=`expr $renew_days \* 86400` # 86400 seconds per day
                if [ $time_diff -ge $renew_time ]; then
                    dyndns_out "Forcing update, last update was more then $renew_days days ago."
                    do_update=yes
                fi
            else
                renew_time=1800 # 30 min
                if [ $time_diff -lt $renew_time ]; then
                    dyndns_out "Ignoring forced update, minimal wait time ist not over yet."
		else
		    do_update=yes
                fi
            fi
        fi
    fi
    if [ "$do_update" != yes ]; then
        dyndns_out "No update needed, ip did not change"
        rm -f $dyn_pid_file
        exit 0
    fi
}

do_update ()
{
    #
    # account disabled?
    #
    [ -f $dyn_disabled ] && abort "Disabled until problem is fixed, see log for mor details ($dyn_disabled or dyndns web interface)"

    aquire_lock
    check_update_needed

    while true; do
        {
            echo "# DynDNS-Provider: $provider"
            echo -n "# DynDNS-Begin: "
            dyndns_date
            echo -e "# DynDNS-End: unfinished\n# DynDNS-Result: unknown"    
            
            try_update
            res=$?
            [ $res -eq $ETRY_TMP ] && comment "All methods failed, restarting after timeout (${dyn_retry_timeout}s)."
        } > $dyn_logfile
        
        if [ $res -eq 0 ]; then
            res_txt=Success
        else
            res_txt=Failed
        fi
        sed -i -e "s/\(# DynDNS-End: \).*/\1`dyndns_date`/;s/\(# DynDNS-Result: \).*/\1$res_txt/" $dyn_logfile
        cat $dyn_logfile >> $dyn_history
        chmod a+r $dyn_logfile
        
        if [ "$dyn_lock" ]; then
            if [ -f "$dyn_lock" ]; then
                rm -f $dyn_lock
                dyndns_log "released lock"
            else
                [ "$dyn_clear_log" ] && dyndns_out "lock already released by someone else"
            fi
            dyn_clear_log=
        fi
        
        case $res in
            $ETRY_TMP)
                sleep $dyn_retry_timeout
                continue
                ;;
            0)
                echo -e "registered_ip='$ip'\nregistered_date='`date +%s`'" > $dyn_runfile
                dyndns_out "Update successful"
                ;;
            $ETRY_PERM)
                dyndns_out "Provider signaled an error, update failed"
                ;;
            $ETRY_NOMETHOD)
                dyndns_out "No methods left to try, update failed"
                ;;
            *)
                dyndns_out "Unknown error '$res', update failed"
                ;;
        esac

        rm -f $dyn_trylog $dyn_trylog.err $dyn_pid_file
        exit $res
    done  
}

cancel_update ()
{
    if [ -f $dyn_pid_file ]; then
        read pid < $dyn_pid_file
        rm -f $dyn_pid_file

        if kill -0 $pid 2> /dev/null; then
            children=`sed -n -e "/^Pid:/h;/^PPid:.*$pid/{g;s/^Pid:[[:space:]]*//p}" /proc/*/status`
            if kill $pid 2> /dev/null; then
                #
                # terminate any pending child processes too
                #
                kill $children
                str="# DynDNS-Cancel: `dyndns_date`"
                echo "$str" >> $dyn_logfile
                cat $dyn_logfile >> $dyn_history
                return 0
            else
                dyndns_out "failed to terminate update (pid: $pid)"
            fi
        fi
    fi
    dyndns_out "No running update found"
    return 1
}

enable_host()
{
    if [ -f $dyn_disabled ]; then
        rm -f $dyn_disabled
        set $provider_update_method
        if [ $# -eq 1 ]; then
            if [ -f $dyn_disabled.$method ]; then
                rm -f $dyn_disabled.$method
            fi
        fi
        str="# DynDNS-Enable: `dyndns_date`"
        echo "$str" >> $dyn_history
        echo "$str" >> $dyn_logfile
    fi
    return 0
}

enable_method()
{
    method=$ip
    if [ -f $dyn_disabled.$method ]; then
        rm -f $dyn_disabled.$method
    fi
}

do_status ()
{
    if [ -f $dyn_disabled ]; then
        echo "$host: disabled"
    else
        if [ -f $dyn_runfile.pid ]; then
            read pid < $dyn_runfile.pid
            echo "$host: update running ($pid)"
        else
            if [ -f $dyn_logfile ]; then
                echo "$host: ok"
            else
                echo "$host: not updated yet"
            fi
        fi
    fi
}

set_defaults ()
{
    #
    # set default values
    #
    conf="/etc/dyndns/dyndns.conf"
    dyn_ver="3.10.5"

    dyn_logdir="/var/log/dyndns"
    dyn_histdir="${dyn_logdir}/history"
    dyn_rundir="/var/run/dyndns"
    dyn_logfile="$dyn_logdir/$host"
    dyn_runfile="$dyn_rundir/$host"
    dyn_history="$dyn_histdir/$host"
    dyn_disabled="${dyn_runfile}.disabled"

    dyn_pid_file="$dyn_rundir/$host.pid"

    dyn_timeout="60"
    dyn_method_timeout=1800
    dyn_retry_timeout=1800

    dyn_trylog=/tmp/dyndns-update.$$.$host

    dyn_clear_log=yes
}

get_ip_for_update ()
{
    
    [ -f /var/run/circuit-$circuit ] || abort "Unable to determine interface for circuit '$circuit'"
    read dev < /var/run/circuit-$circuit
    
    if [ "$dev" = "pppoe" ]
    then
        dev=ppp0
    fi

    case "$extip" in
    no)
        ip=`ip a s $dev | sed -ne 's/^[[:space:]]*inet[[:space:]]\([0-9\.]*\).*/\1/p'`
        ;;
    *)
        # find device on every default route on every routing table with:
        # ip route show table all | grep -q "^default via .*dev $circ_dev "
        # but the result is never used?
        #ip r s | grep -q "^default via.*dev $circ_dev" || abort "Can't determine external id for non default route device '$circ_dev' of circuit '$circuit'"
        get_external_ip $extip $dev
        ;;
    esac
    
    if [ "$ip" ]; then
        set $ip
        [ $# -eq 1 ] || abort "Interface '$dev' of circuit '$circuit' has multiple ip addresses ($*)"
    else
        abort "Unable to determine ip address of interface '$dev' for circuit '$circuit'"
    fi

    if [ -f $dyn_rundir/inv_IP ]; then
        if [ -f $dyn_rundir/fake_ip ]; then
            read octet < $dyn_rundir/fake_ip
        else 
            octet=1
        fi 
        octet=`expr \( $octet + 1 \) % 254`
        echo $octet > $dyn_rundir/fake_ip

        ip=${ip%\.*}.$octet
    fi
}

get_config ()
{
    #
    # get config information
    #
    conf_line="`grep \"^$host[[:space:]].*[[:space:],]$circuit[[:space:],]\" $conf`"
    [ "$conf_line" ] || abort "Missing config line for host '$host,$circuit' in config file '$conf'"
    
    set -- $conf_line
    [ $# -eq 7 -o $# -eq 8 ] || abort "Invalid config line for host '$host' in config file '$conf'"
    
    # $1 - host, $3 - circuit: we already have that
    provider="$2"
    user="$4"
    pass="$5"
    extip="$6"
    renew_days="$7"
    updatehost="$8"
    
    if [ $renew_days = '-' ]
    then
      renew_days=''
    fi
    
    [ -f $dyn_rundir/inv_user ] && user="inv_$user"
    [ -f $dyn_rundir/inv_pass ] && user="inv_$pass"
    [ -f $dyn_rundir/inv_host ] && user="inv_$host"

    #
    # divide hostname in subdomain and domain
    #
    subdom=${host%.*.*}
    domain=${host#$subdom.}

    # 
    # get ip needed to calculate strings correctly while sourcing provider files
    #
    [ "$action" = update ] && get_ip_for_update

    #
    # read config for provider
    #
    . /etc/dyndns/provider.$provider
    [ "$provider_update_method" ] || abort "Empty update type in /etc/dyndns/provider.$provider."
    if [ ! -f /etc/dyndns/allow_ssl ]; then
        disable_method https
        disable_method https_noauth
        [ "$provider_update_method" ] || abort "Usage of ssl disabled, but ssl was the only update method"
    fi
}

#
# check parameters
#
prog=${0##*/}

action=update
args=2
while [ "$1" ]; do
    case "$1" in
        -x | -vx)
            deb_flag=$1
            ;;
        -f)
            dyn_force=yes
            ;;
        -ff)
            dyn_force_always=yes
            ;;
        -l) if [ "$2" ]; then
                dyn_lock=$2
                shift
            else
                usage
            fi
            ;;
        -s) syslog=yes ;;
        update | enable_method)
            action=$1
            args=2
            ;;
        enable | cancel | status)
            action=$1
            args=1
            ;;
        *)  break ;;
    esac
    shift
done

if [ $# -ne $args ]; then
    if [ $action != update -o $# -ne 1 ]; then
	usage
    fi
fi

if [ "$deb_flag" ] ; then
    file=/var/tmp/dyndns-update_`date +%y-%m-%d_%H-%M-%S`_${action}_${1}_trace.log
    exec 2> $file
    set $deb_flag
fi

host="$1"
circuit="$2"
if [ ! "$circuit" ]; then
    # no circuit specified, lookup default route circuit
    if [ -f /etc/default-route-interface ]; then 
	read dev < /etc/default-route-interface
	for circ in /var/run/circuit-*; do
	    [ ! -f $circ ] && break
	    read cdev < $circ
	    if [ "$cdev" = "$dev" ]; then
		circuit=${circ##/var/run/circuit-}
		break
	    fi
	done
    elif [ -f /var/run/circuit-static ]; then
        circuit=static
    elif [ -f /var/run/circuit-dhcp ]; then
        circuit=dhcp
    else
        abort "Unknown circuit methode, not able to lookup default route circuit"
    fi
    [ "$circuit" ] || abort "Unable to lookup default route circuit"
fi

set_defaults
get_config

case $action in
    update) do_update ;;
    enable) enable_host ;;
    enable_method) enable_method ;;
    cancel) cancel_update ;;
    status) do_status ;;
esac

exit $?
